#include "statebut.h"
#include <windowsx.h>

void TStateButton::StateChange(void)
{
	HDC hdc = GetDC(HWindow);

	if(Chars.State == SB_ON)
		DrawOn(hdc);
	else
		DrawOff(hdc);

	ReleaseDC(HWindow, hdc);
}

TStateButton::TStateButton(PTWindowsObject AParent, int AnId,
		LPSTR ATitle, int X, int Y, int W, int H,
		PTModule	ATModule)
		: TControl(AParent, AnId, ATitle, X, Y, W, H,
      	ATModule)
{
	Config();
}

TStateButton::TStateButton(PTWindowsObject AParent, int ResourceId,
		PTModule ATModule)
		: TControl(AParent, ResourceId, ATModule)
{
	Config();
}

TStateButton::Config()
{
	Chars.State = SB_OFF;
	Chars.Focus = FALSE;
	Chars.Capture = FALSE;
	Chars.Space = FALSE;
	Chars.LastIn = FALSE;
   Chars.Changing = FALSE;
}

void TStateButton::SetupWindow()
{
	TControl::SetupWindow();

	GetClientRect(HWindow, &ClientRect);
}

void TStateButton::WMSetFocus(RTMessage Msg)
{
	Chars.Focus = TRUE;

	HDC hdc = GetDC(HWindow);

   ChangeFocus(hdc);

   ReleaseDC(HWindow, hdc);
}

void TStateButton::WMKillFocus(RTMessage Msg)
{
	if(Chars.Capture)
		ReleaseCapture();

	Chars.Focus = FALSE;

	HDC hdc = GetDC(HWindow);

	ChangeFocus(hdc);

   ReleaseDC(HWindow, hdc);
}

void TStateButton::WMLButtonDown(RTMessage Msg)
{
	if(Chars.Space || Chars.Capture)
		return;

	Chars.Capture = TRUE;
	Chars.Changing = TRUE;
	Chars.LastIn = TRUE;

   Toggle();

	SetCapture(HWindow);
}

void TStateButton::WMLButtonUp(RTMessage Msg)
{
	if(Chars.Space)
		return;

	ReleaseCapture();
	Chars.Capture = FALSE;
   Chars.Changing = FALSE;

	TMessage ToSend;
	ToSend.Message = WM_COMMAND;
	ToSend.Receiver = Parent->HWindow;
	ToSend.WParam = GetId();
	ToSend.LP.Lo = (int)HWindow;
	ToSend.LP.Hi = BN_CLICKED;

	if(PtInRect(&ClientRect, *(POINT*)&Msg.LParam))
		SendMessage(ToSend.Receiver, ToSend.Message,
			ToSend.WParam, ToSend.LParam);

	SetState(Chars.State);
}

void TStateButton::WMKeyDown(RTMessage Msg)
{
	if(Chars.Space || Chars.Capture)
		return;

	if(Msg.WParam == VK_SPACE)
		{
		Chars.Space = TRUE;
      Chars.Changing = TRUE;

      Toggle();
		}
}

void TStateButton::WMKeyUp(RTMessage Msg)
{
	if(!Chars.Space || Chars.Capture)
		return;

	if(Msg.WParam == VK_SPACE)
		{
		Chars.Space = FALSE;
		Chars.Changing = FALSE;

      SetState(Chars.State);

		TMessage ToSend;
		ToSend.Message = WM_COMMAND;
		ToSend.Receiver = Parent->HWindow;
		ToSend.WParam = GetId();
		ToSend.LP.Lo = (int)HWindow;
		ToSend.LP.Hi = BN_CLICKED;

		SendMessage(ToSend.Receiver, ToSend.Message,
			ToSend.WParam, ToSend.LParam);
		}
}

void TStateButton::WMMouseMove(RTMessage Msg)
{
	if(!Chars.Capture)
		return;

	if(PtInRect(&ClientRect, *(POINT*)&Msg.LParam))
		{
		if(!Chars.LastIn)
			{
			Chars.LastIn = TRUE;
         Toggle();
         }
		}
	else
		{
		if(Chars.LastIn)
			{
			Chars.LastIn = FALSE;
			Toggle();
         }
		}
}

void TStateButton::WMGetDlgCode(RTMessage Msg)
{
	Msg.Result = DLGC_RADIOBUTTON;
}

void TStateButton::Paint(HDC PaintDC, PAINTSTRUCT _FAR & PaintInfo)
{
	if(Chars.State == SB_ON)
		DrawOn(PaintDC);
	else
   	DrawOff(PaintDC);
}

void TStateButton::WMSize(RTMessage Msg)
{
	GetClientRect(HWindow, &ClientRect);
   TWindow::WMSize(Msg);
}

void TStateButton::DrawOff(HDC DrawDC)
{
}

void TStateButton::DrawOn(HDC DrawDC)
{
}

void TStateButton::ChangeFocus(HDC DrawDC)
{
	DrawFocusRect(DrawDC, &ClientRect);
}

void TStateButton::GetWindowClass(WNDCLASS& WndClass)
{
	TWindow::GetWindowClass(WndClass);
}